<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;
use \Doctrine\ORM\EntityManager;

class CombinacoesProdutos{
    
    protected $entityManager; 

	public function __construct(EntityManager $entityManager)  {
		ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '256M');
		$this->entityManager = $entityManager;
	}
	
	private function setParams ($conditions){
		$params = array();
		if(count($conditions)){
			$params = array(
				'NRORG'=>$conditions['NRORG'],
				'DTINI'=>$conditions['DTVENDAINI'],
				'DTFIM'=>$conditions['DTVENDAFIN'],
				'PRODINI'=>$conditions['CDARVPRODI'],
				'PRODFIM'=>$conditions['CDARVPRODF'],
				'TCDFILIAL'=>$conditions['TCDFILIAL'],
				'CDARVPROD'=>$conditions['CDARVPROD'],
				'TCDARVPROD'=>$conditions['TCDARVPROD'],
				'CDFILIAL'=>$conditions['CDFILIAL'],
			);
		}
		return $params;
	}
	
	private function setParamsQueryFilha ($conditions,$rowPai){
		$params = array();
		unset($conditions['CDFILIAL']);
		unset($conditions['TCDARVPROD']);
		unset($conditions['TCDFILIAL']);
		if(count($conditions)){
			$params = array_merge($conditions,['CDPRODPAI'=>$rowPai['CDPRODPAI'],'CDFILIAL'=>$rowPai['CDFILIAL']]);
		}
		return $params;
	}
	
	public function find ($conditions){
	    $conditions = $this->getDataInterval($conditions);
	    $conditions = $this->ajustaIntervaloDeProdutos($conditions);
	    
	    return $this->prepararResultados($this->setParams($conditions));
	}
	
	private function prepararResultados ($params){
		$resultadoQueryPai = $this->queryPrincipal($params);
		$merge = [];
		if(count($resultadoQueryPai)){
			foreach ($resultadoQueryPai as $value) {
				$resultadoQueryFilha = $this->queryFilha($this->setParamsQueryFilha($params,$value));
				foreach ($resultadoQueryFilha as $valueFilho) {
					$valueFilho['TAXA'] = $valueFilho['QUANTPRODFIL'] / $value['QUANTPRODPAI'] * 100;
					$value['LABELNMPRODPAI'] = $value['CDPRODPAI'].' - '
					.$value['NMPRODPAI'].' | Valor Vendido: '.$this->numberFormat($value['TOTALVENDA'])
					.'| Nr. de Cupons: '.$this->numberFormat($value['NRCUPONS'])
					.' | Qtde. Vendida: '.$this->numberFormat($value['QUANTPRODPAI'])
					.' | Valor Médio Cupom: '.$this->numberFormat($value['TOTALVENDA']/$value['NRCUPONS']);
					$valueFilho['LABELPRODFILHO'] = $valueFilho['CDSUBPRODFIL'].' - '.$valueFilho['NMSUBPRODFIL'];
					$nomeFilial = $value['CDFILIAL'].' - '.$value['NMFILIAL'];
					$value['LABELFILIAL'] = $nomeFilial;
					$merge[] = array_merge($value,$valueFilho);
				}
			}
		}
		return $merge;
	}
	
	private function numberFormat ($number){
		return number_format($number, 2, ',', '.');
	}

	private function queryPrincipal ($params){
		return $this->entityManager->getConnection()->executeQuery(self::QUERY_PRINCIPAL,$params)->fetchAll();
	}
	
	private function queryFilha ($params){
		return $this->entityManager->getConnection()->executeQuery(self::QUERY_FILHA,$params)->fetchAll();
	}
	
	private function getDataInterval ($conditions){
	    $conditions['DTVENDAINI'] = $conditions['DTMOVIMENTO'][0];
        $conditions['DTVENDAFIN'] = $conditions['DTMOVIMENTO'][1];
        unset($conditions['DTMOVIMENTO']);
        return $conditions;
	}
	
	private function ajustaIntervaloDeProdutos ($conditions){
	
	    if(!isset($conditions['CDARVPRODI'])){
            $conditions['CDARVPRODI'] = null;
            $conditions['CDARVPRODF'] = null;
        }
        if(!isset($conditions['CDARVPRODF']) && is_null($conditions['CDARVPRODI'])){
            $conditions['CDARVPRODF'] = $conditions['CDARVPRODI'];
        }
        $flag = empty($conditions['CDARVPROD']) && is_null($conditions['CDARVPRODI']);
        $conditions['TCDARVPROD'] = $flag ? 'T':'N';
	    return $conditions;
	}
	
	const QUERY_PRINCIPAL = "
		SELECT IT.CDPRODUTO AS CDPRODPAI,
		  VE.CDFILIAL,
		  VE.NMFILIAL,
		  PR.NMPRODPAI,
		  PR.CDARVPROD,
		  PR.CDBARPRODPAI,
		  VE.NRORG,
		  COUNT(DISTINCT VE.CDFILIAL
		  ||VE.CDCAIXA
		  ||VE.NRSEQVENDA)                                                                                                         AS NRCUPONS,
		  SUM(IT.QTPRODVEND)                                                                                                       AS QUANTPRODPAI,
		  SUM((TRUNC(IT.QTPRODVEND * (IT.VRUNITVEND + NVL(IT.VRUNITVENDCL,0)),2)) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0)) AS TOTALVENDA
		FROM
		  (SELECT DISTINCT V.CDCAIXA,
		    V.CDFILIAL,
		    V.NRORG,
		    FI.NMFILIAL,
		    V.NRSEQVENDA
		  FROM VENDA V,
		    (SELECT DISTINCT L.*,
		      C.CDCAIXA
		    FROM CAIXA C,
		      (SELECT DISTINCT F.*,
		        L.CDLOJA
		      FROM LOJA L,
		        (SELECT DISTINCT CDFILIAL,
		          NMFILIAL
		        FROM FILIAL
		        WHERE (INSTR(:CDFILIAL, CDFILIAL) <> 0)
		        OR (:TCDFILIAL                     = 'T')
		        ) F
		      WHERE L.CDFILIAL = F.CDFILIAL
		      ) L
		    WHERE C.CDFILIAL = L.CDFILIAL
		    AND C.CDLOJA     = L.CDLOJA
		    ) FI
		  WHERE V.CDCAIXA   = FI.CDCAIXA
		  AND V.CDLOJA      = FI.CDLOJA
		  AND V.CDFILIAL    = FI.CDFILIAL
		  AND V.NRORG       = :NRORG
		  AND V.IDSITUVENDA = 'O'
		  AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIM
		  ) VE,
		  (SELECT NMPRODUTO AS NMPRODPAI,
		    CDARVPROD,
		    CDBARPRODUTO AS CDBARPRODPAI,
		    CDPRODUTO
		  FROM PRODUTO
		  WHERE ((CDARVPROD BETWEEN :PRODINI AND :PRODFIM)
		  OR (INSTR(:CDARVPROD, CDARVPROD) <> 0) OR :TCDARVPROD = 'T')
		  ) PR,
		  ITEMVENDA IT
		WHERE IT.CDPRODUTO = PR.CDPRODUTO
		AND IT.NRSEQVENDA  = VE.NRSEQVENDA
		AND IT.CDFILIAL    = VE.CDFILIAL
		AND IT.CDCAIXA     = VE.CDCAIXA
		AND IT.NRORG       = :NRORG
		GROUP BY VE.CDFILIAL,
		  VE.NMFILIAL,
		  IT.CDPRODUTO,
		  PR.NMPRODPAI,
		  PR.CDARVPROD,
		  PR.CDBARPRODPAI,
		  VE.NRORG
		ORDER BY VE.CDFILIAL,
		  PR.NMPRODPAI
	";
	
	const QUERY_FILHA = "
		SELECT IT.CDPRODUTO                                                                                                                            AS CDSUBPRODFIL,
		  PR.NMPRODUTO                                                                                                                                 AS NMSUBPRODFIL,
		  PR.CDBARPRODUTO                                                                                                                              AS CDBARPRODFIL,
		  SUM(IT.QTPRODVEND)                                                                                                                           AS QUANTPRODFIL,
		  SUM((TRUNC(IT.QTPRODVEND * (IT.VRUNITVEND + NVL(IT.VRUNITVENDCL,0)),2)) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0))                     AS TOTALVENDFIL,
		  SUM((TRUNC(IT.QTPRODVEND * (IT.VRUNITVEND + NVL(IT.VRUNITVENDCL,0)),2)) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0) - (IP.VRIMPOPRODIT)) AS VRLIQ
		FROM
		  (SELECT DISTINCT V.CDCAIXA,
		    V.CDFILIAL,
		    V.NRSEQVENDA,
		    V.NRORG
		  FROM VENDA V,
		    (SELECT DISTINCT L.*,
		      C.CDCAIXA
		    FROM CAIXA C,
		      (SELECT DISTINCT F.*,
		        L.CDLOJA
		      FROM LOJA L,
		        (SELECT DISTINCT CDFILIAL FROM FILIAL WHERE CDFILIAL = :CDFILIAL
		        ) F
		      WHERE L.CDFILIAL = F.CDFILIAL
		      ) L
		    WHERE C.CDFILIAL = L.CDFILIAL
		    AND C.CDLOJA     = L.CDLOJA
		    ) FI
		  WHERE V.CDCAIXA   = FI.CDCAIXA
		  AND V.CDLOJA      = FI.CDLOJA
		  AND V.CDFILIAL    = FI.CDFILIAL
		  AND V.NRORG = :NRORG
		  AND V.IDSITUVENDA = 'O'
		  AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIM
		  AND EXISTS
		    (SELECT IT.CDFILIAL
		    FROM ITEMVENDA IT
		    WHERE (IT.CDPRODUTO = :CDPRODPAI)
		    AND (IT.CDFILIAL    = V.CDFILIAL)
		    AND (IT.NRSEQVENDA  = V.NRSEQVENDA)
		    AND (IT.CDCAIXA     = V.CDCAIXA)
		    )
		  ) VE,
		  PRODUTO PR,
		  ITVENDAIMPOS IP,
		  ITEMVENDA IT
		WHERE (VE.CDFILIAL   = IT.CDFILIAL )
		AND (VE.CDCAIXA      = IT.CDCAIXA )
		AND (VE.NRSEQVENDA   = IT.NRSEQVENDA )
		AND (IT.CDFILIAL     = IP.CDFILIAL )
		AND (IT.NRSEQVENDA   = IP.NRSEQVENDA )
		AND (IT.CDCAIXA      = IP.CDCAIXA )
		AND (IT.NRSEQUITVEND = IP.NRSEQUITVEND)
		AND (IT.CDPRODUTO    = PR.CDPRODUTO )
		AND (IT.CDPRODUTO   <> :CDPRODPAI )
		AND (VE.NRORG = IT.NRORG)
		GROUP BY IT.CDPRODUTO,
		  PR.NMPRODUTO,
		  PR.CDBARPRODUTO
		ORDER BY 4 DESC
	";
}